@@ -9,11 +9,11 @@
 
 #include "kosm_export.h"
 #include "internal.h"
+#include "languages.h"
 #include "stringpool.h"
 
 #include <QByteArray>
 #include <QDebug>
-#include <QLocale>
 #include <QString>
 
 #include <cstdint>
@@ -57,17 +57,17 @@ public:
         }
     }
 
-    constexpr inline bool isValid() const
+    [[nodiscard]] constexpr inline bool isValid() const
     {
         return latitude != std::numeric_limits<uint32_t>::max() && longitude != std::numeric_limits<uint32_t>::max();
     }
-    constexpr inline bool operator==(Coordinate other) const
+    [[nodiscard]] constexpr inline bool operator==(Coordinate other) const
     {
         return latitude == other.latitude && longitude == other.longitude;
     }
 
     /** Z-order curve value for this coordinate. */
-    constexpr inline uint64_t z() const
+    [[nodiscard]] constexpr inline uint64_t z() const
     {
         uint64_t z = 0;
         for (int i = 0; i < 32; ++i) {
@@ -77,11 +77,11 @@ public:
         return z;
     }
 
-    constexpr inline double latF() const
+    [[nodiscard]] constexpr inline double latF() const
     {
         return (latitude / 10'000'000.0) - 90.0;
     }
-    constexpr inline double lonF() const
+    [[nodiscard]] constexpr inline double lonF() const
     {
         return (longitude / 10'000'000.0) - 180.0;
     }
@@ -99,34 +99,34 @@ public:
         : min(c1)
         , max(c2)
     {}
-    constexpr inline bool isValid() const
+    [[nodiscard]] constexpr inline bool isValid() const
     {
         return min.isValid() && max.isValid();
     }
-    constexpr inline bool operator==(BoundingBox other) const
+    [[nodiscard]] constexpr inline bool operator==(BoundingBox other) const
     {
         return min == other.min && max == other.max;
     }
 
-    constexpr inline uint32_t width() const
+    [[nodiscard]] constexpr inline uint32_t width() const
     {
         return max.longitude - min.longitude;
     }
-    constexpr inline uint32_t height() const
+    [[nodiscard]] constexpr inline uint32_t height() const
     {
         return max.latitude - min.latitude;
     }
 
-    constexpr inline double widthF() const
+    [[nodiscard]] constexpr inline double widthF() const
     {
         return width() / 10'000'000.0;
     }
-    constexpr inline double heightF() const
+    [[nodiscard]] constexpr inline double heightF() const
     {
         return height() / 10'000'000.0;
     }
 
-    constexpr inline Coordinate center() const
+    [[nodiscard]] constexpr inline Coordinate center() const
     {
         return Coordinate(min.latitude + height() / 2, min.longitude + width() / 2);
     }
@@ -135,7 +135,7 @@ public:
     Coordinate max;
 };
 
-constexpr inline BoundingBox unite(BoundingBox bbox1, BoundingBox bbox2)
+[[nodiscard]] constexpr inline BoundingBox unite(BoundingBox bbox1, BoundingBox bbox2)
 {
     if (!bbox1.isValid()) {
         return bbox2;
@@ -151,24 +151,24 @@ constexpr inline BoundingBox unite(BoundingBox bbox1, BoundingBox bbox2)
     return ret;
 }
 
-constexpr inline bool intersects(BoundingBox bbox1, BoundingBox bbox2)
+[[nodiscard]] constexpr inline bool intersects(BoundingBox bbox1, BoundingBox bbox2)
 {
     return !(bbox2.min.latitude > bbox1.max.latitude || bbox2.max.latitude < bbox1.min.latitude
         || bbox2.min.longitude > bbox1.max.longitude || bbox2.max.longitude < bbox1.min.longitude);
 }
 
-constexpr inline bool contains(BoundingBox bbox, Coordinate coord)
+[[nodiscard]] constexpr inline bool contains(BoundingBox bbox, Coordinate coord)
 {
     return bbox.min.latitude <= coord.latitude && bbox.max.latitude >= coord.latitude
         && bbox.min.longitude <= coord.longitude && bbox.max.longitude >= coord.longitude;
 }
 
-constexpr inline uint32_t latitudeDistance(BoundingBox bbox1, BoundingBox bbox2)
+[[nodiscard]] constexpr inline uint32_t latitudeDistance(BoundingBox bbox1, BoundingBox bbox2)
 {
     return bbox1.max.latitude < bbox2.min.latitude ? bbox2.min.latitude - bbox1.max.latitude : bbox1.min.latitude - bbox2.max.latitude;
 }
 
-constexpr inline uint32_t longitudeDifference(BoundingBox bbox1, BoundingBox bbox2)
+[[nodiscard]] constexpr inline uint32_t longitudeDifference(BoundingBox bbox1, BoundingBox bbox2)
 {
     return bbox1.max.longitude < bbox2.min.longitude ? bbox2.min.longitude - bbox1.max.longitude : bbox1.min.longitude - bbox2.max.longitude;
 }
@@ -182,19 +182,23 @@ class TagKey : public StringKey {};
 class Tag {
 public:
     Tag() = default;
+    inline Tag(TagKey _key, QByteArray &&_value)
+        : key(_key)
+        , value(std::move(_value))
+    {}
     Tag(const Tag&) = default;
     Tag(Tag &&Tag) noexcept = default;
     Tag& operator=(const Tag &other) = default;
     Tag& operator=(Tag &&other) noexcept = default;
 
-    inline constexpr bool operator<(const Tag &other) const { return key < other.key; }
+    [[nodiscard]] inline constexpr bool operator<(const Tag &other) const { return key < other.key; }
 
     TagKey key;
     QByteArray value;
 };
 
-inline constexpr bool operator<(const Tag &lhs, TagKey rhs) { return lhs.key < rhs; }
-inline constexpr bool operator<(TagKey lhs, const Tag &rhs) { return lhs < rhs.key; }
+[[nodiscard]] inline constexpr bool operator<(const Tag &lhs, TagKey rhs) { return lhs.key < rhs; }
+[[nodiscard]] inline constexpr bool operator<(TagKey lhs, const Tag &rhs) { return lhs < rhs.key; }
 
 /** An OSM node. */
 class KOSM_EXPORT Node {
@@ -208,15 +212,15 @@ public:
     Node& operator=(const Node &other) = default;
     Node& operator=(Node &&other) noexcept
     {
-        id = std::move(other.id);
-        coordinate = std::move(other.coordinate);
+        id = other.id;
+        coordinate = other.coordinate;
         std::swap(tags, other.tags);
         return *this;
     }
 
-    constexpr inline bool operator<(const Node &other) const { return id < other.id; }
+    [[nodiscard]] constexpr inline bool operator<(const Node &other) const { return id < other.id; }
 
-    QString url() const;
+    [[nodiscard]] QString url() const;
 
     Id id;
     Coordinate coordinate;
@@ -235,8 +239,8 @@ public:
     Way& operator=(const Way &other) = default;
     Way& operator=(Way &&other) noexcept
     {
-        id = std::move(other.id);
-        bbox = std::move(other.bbox);
+        id = other.id;
+        bbox = other.bbox;
         std::swap(nodes, other.nodes);
         std::swap(tags, other.tags);
         return *this;
@@ -277,11 +281,11 @@ private:
 /** A member in a relation. */
 class Member {
 public:
-    inline bool operator==(const Member &other) const { return id == other.id && m_roleAndType == other.m_roleAndType; }
+    [[nodiscard]] inline bool operator==(const Member &other) const { return id == other.id && m_roleAndType == other.m_roleAndType; }
 
     Id id;
 
-    constexpr inline Role role() const
+    [[nodiscard]] constexpr inline Role role() const
     {
         return Role(m_roleAndType.get());
     }
@@ -290,7 +294,7 @@ public:
         m_roleAndType.set(role.name());
     }
 
-    constexpr inline Type type() const
+    [[nodiscard]] constexpr inline Type type() const
     {
         return static_cast<Type>(m_roleAndType.tag());
     }
@@ -315,16 +319,16 @@ public:
     Relation& operator=(const Relation &other) = default;
     Relation& operator=(Relation &&other) noexcept
     {
-        id = std::move(other.id);
-        bbox = std::move(other.bbox);
+        id = other.id;
+        bbox = other.bbox;
         std::swap(members, other.members);
         std::swap(tags, other.tags);
         return *this;
     }
 
-    constexpr inline bool operator<(const Relation &other) const { return id < other.id; }
+    [[nodiscard]] constexpr inline bool operator<(const Relation &other) const { return id < other.id; }
 
-    QString url() const;
+    [[nodiscard]] QString url() const;
 
     Id id;
     mutable BoundingBox bbox;
@@ -337,27 +341,27 @@ class KOSM_EXPORT DataSet {
 public:
     explicit DataSet();
     DataSet(const DataSet&) = delete;
-    DataSet(DataSet &&other);
+    DataSet(DataSet &&other) noexcept;
     ~DataSet();
 
     DataSet& operator=(const DataSet&) = delete;
-    DataSet& operator=(DataSet &&);
+    DataSet& operator=(DataSet &&) noexcept;
 
     /** Find a node by its id.
      *  @returns @c nullptr if the node doesn't exist.
      */
-    const Node* node(Id id) const;
+    [[nodiscard]] const Node* node(Id id) const;
 
     /** Find a way by its id.
      *  @returns @c nullptr if the way doesn't exist.
      */
-    const Way* way(Id id) const;
-    Way* way(Id id);
+    [[nodiscard]] const Way* way(Id id) const;
+    [[nodiscard]] Way* way(Id id);
 
     /** Find a relation by its id.
      *  @returns @c nullptr if the relation doesn't exist.
      */
-    const Relation* relation(Id id) const;
+    [[nodiscard]] const Relation* relation(Id id) const;
 
     void addNode(Node &&node);
     void addWay(Way &&way);
@@ -367,7 +371,7 @@ public:
      *  If no key exists, an empty/invalid/null key is returned.
      *  Use this for tag lookup, not for creating/adding tags.
      */
-    TagKey tagKey(const char *keyName) const;
+    [[nodiscard]] TagKey tagKey(const char *keyName) const;
 
     /** Create a tag key for the given tag name. If none exist yet a new one is created.
      *  Use this for creating tags, not for lookup, prefer tagKey() for that.
@@ -375,19 +379,19 @@ public:
      *  instance and thus can be used without requiring a copy. If the memory is transient
      *  the string is copied if needed, and released in the DataSet destructor.
      */
-    TagKey makeTagKey(const char *keyName, StringMemory keyMemOpt = StringMemory::Transient);
+    [[nodiscard]] TagKey makeTagKey(const char *keyName, StringMemory keyMemOpt = StringMemory::Transient);
 
     /** Looks up a role name key.
      *  @see tagKey()
      */
-    Role role(const char *roleName) const;
+    [[nodiscard]] Role role(const char *roleName) const;
     /** Creates a role name key.
      *  @see makeTagKey()
      */
-    Role makeRole(const char *roleName, StringMemory memOpt = StringMemory::Transient);
+    [[nodiscard]] Role makeRole(const char *roleName, StringMemory memOpt = StringMemory::Transient);
 
     /** Create a unique id for internal use (ie. one that will not clash with official OSM ids). */
-    Id nextInternalId() const;
+    [[nodiscard]] Id nextInternalId() const;
 
     std::vector<Node> nodes;
     std::vector<Way> ways;
@@ -403,7 +407,7 @@ private:
 
 /** Returns the tag value for @p key of @p elem. */
 template <typename Elem>
-inline QByteArray tagValue(const Elem& elem, TagKey key)
+[[nodiscard]] inline QByteArray tagValue(const Elem& elem, TagKey key)
 {
     const auto it = std::lower_bound(elem.tags.begin(), elem.tags.end(), key);
     if (it != elem.tags.end() && (*it).key == key) {
@@ -417,7 +421,7 @@ inline QByteArray tagValue(const Elem& elem, TagKey key)
  *  Where possible avoid this in favor of tagValue().
  */
 template <typename Elem>
-inline QByteArray tagValue(const Elem& elem, const char *keyName)
+[[nodiscard]] inline QByteArray tagValue(const Elem& elem, const char *keyName)
 {
     const auto it = std::find_if(elem.tags.begin(), elem.tags.end(), [keyName](const auto &tag) { return std::strcmp(tag.key.name(), keyName) == 0; });
     if (it != elem.tags.end()) {
@@ -430,30 +434,17 @@ inline QByteArray tagValue(const Elem& elem, const char *keyName)
  *  @warning This is slow due to doing a linear search and string comparissons.
  */
 template <typename Elem>
-inline QByteArray tagValue(const Elem& elem, const QLocale &locale, const char *keyName)
+[[nodiscard]] inline QByteArray tagValue(const Elem& elem, const OSM::Languages &languages, const char *keyName)
 {
-    QByteArray key(keyName);
-    key.push_back(':');
-    const auto baseLen = key.size();
-    for (const auto &lang : locale.uiLanguages()) {
-        key.resize(baseLen);
-        key.append(lang.toUtf8());
-        const auto it = std::find_if(elem.tags.begin(), elem.tags.end(), [key](const auto &tag) { return std::strcmp(tag.key.name(), key.constData()) == 0; });
-        if (it != elem.tags.end()) {
-            return (*it).value;
-        }
-
-        const auto idx = lang.indexOf(QLatin1Char('-'));
-        if (idx > 0) {
-            key.resize(baseLen);
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-            key.append(lang.leftRef(idx).toUtf8());
-#else
-            key.append(QStringView(lang).left(idx).toUtf8());
-#endif
-            const auto it = std::find_if(elem.tags.begin(), elem.tags.end(), [key](const auto &tag) { return std::strcmp(tag.key.name(), key.constData()) == 0; });
-            if (it != elem.tags.end()) {
-                return (*it).value;
+    const auto keyLen = std::strlen(keyName);
+    for (const auto &lang : languages.languages) {
+        for (const auto &tag : elem.tags) {
+            if (std::strlen(tag.key.name()) != keyLen + lang.size() + 1) {
+                continue;
+            }
+            if (std::strncmp(tag.key.name(), keyName, keyLen) == 0 && tag.key.name()[keyLen] == ':'
+                && std::strncmp(tag.key.name() + keyLen + 1, lang.c_str(), lang.size()) == 0) {
+                return tag.value;
             }
         }
     }
@@ -465,10 +456,10 @@ inline QByteArray tagValue(const Elem& elem, const QLocale &locale, const char *
     }
 
     // check if there is at least one in any language we can use
-    key.resize(baseLen);
-    const auto it = std::find_if(elem.tags.begin(), elem.tags.end(), [key, baseLen](const auto &tag) {
-        return std::strncmp(tag.key.name(), key.constData(), baseLen) == 0
-            && std::strlen(tag.key.name()) == key.size() + 2; // primitive check whether this is a plausible language rather than some other qualifier
+    const auto it = std::find_if(elem.tags.begin(), elem.tags.end(), [keyName, keyLen](const auto &tag) {
+        return std::strlen(tag.key.name()) == keyLen + 3 // primitive check whether this is a plausible language rather than some other qualifier
+            && std::strncmp(tag.key.name(), keyName, keyLen) == 0
+            && tag.key.name()[keyLen] == ':';
     });
     if (it != elem.tags.end()) {
         return (*it).value;
@@ -490,10 +481,9 @@ inline void setTag(Elem &elem, Tag &&tag)
 
 /** Inserts a new tag, or updates an existing one. */
 template <typename Elem>
-inline void setTagValue(Elem &elem, TagKey key, const QByteArray &value)
+inline void setTagValue(Elem &elem, TagKey key, QByteArray &&value)
 {
-    Tag tag{ key, value };
-    setTag(elem, std::move(tag));
+    setTag(elem, Tag(key, std::move(value)));
 }
 
 /** Removes a tag from the given element. */
@@ -507,7 +497,7 @@ inline void removeTag(Elem &elem, TagKey key)
 }
 
 template <typename Elem>
-inline bool operator<(const Elem &elem, Id id)
+[[nodiscard]] inline bool operator<(const Elem &elem, Id id)
 {
     return elem.id < id;
 }
@@ -517,4 +507,6 @@ inline bool operator<(const Elem &elem, Id id)
 KOSM_EXPORT QDebug operator<<(QDebug debug, OSM::Coordinate coord);
 KOSM_EXPORT QDebug operator<<(QDebug debug, OSM::BoundingBox bbox);
 
+Q_DECLARE_METATYPE(OSM::BoundingBox)
+
 #endif // OSM_DATATYPES_H
